/*
 * Copyright (C) 2016 Michal Kosciesza <michal@mkiol.net>
 * 
 * This file is part of Kaktus.
 * 
 * Kaktus is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Kaktus is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Kaktus.  If not, see <http://www.gnu.org/licenses/>.
 */

import bb.cascades 1.3

Page {
    id: root
    
    property string title
    property string contents
    property string entryId
    property string offlineUrl
    property string onlineUrl
    property bool stared
    property int read
    property int index
    property int feedindex
    property bool cached
    property bool broadcast
    property string annotations
    property bool liked
    
    property bool isLight: utils.isLight()
    property bool empty: contents.length == 0;
    
    function disconnectSignals() {
    }
    
    function check() {        
        // Not allowed while Syncing
        if (dm.busy || fetcher.busy || dm.removerBusy) {
            notification.show(qsTr("Please wait until current task is complete."));
            return false;
        }
        
        // Entry not cached and offline mode enabled
        if (settings.offlineMode && !cached) {
            notification.show(qsTr("Offline version not available."));
            return false;
        }
        
        // Switch to Offline mode if no network
        if (!settings.offlineMode && !dm.online) {
            if (cached) {
                // Entry cached
                notification.show(qsTr("Network connection is unavailable.\nSwitching to Offline mode."));
                settings.offlineMode = true;
            } else {
                // Entry not cached
                notification.show(qsTr("Network connection is unavailable."));
                return false;
            }
        }
        
        return true;
    }
    
    function openEntryInViewer() {
        nav.pop();
        var obj = webPreviewPage.createObject();
        obj.onlineUrl = onlineUrl;
        obj.offlineUrl = offlineUrl;
        obj.title = title;
        obj.stared = stared;
        obj.read = read;
        obj.cached = cached;
        obj.index = index;
        obj.broadcast = broadcast;
        obj.liked = liked;
        obj.annotations = annotations;
        nav.push(obj);
        obj.start();
    }
    
    function openInExaternalBrowser() {
        var link = settings.offlineMode ? offlineUrl : onlineUrl;
        notification.show(qsTr("Launching an external browser..."));
        utils.launchBrowser(link);
    }
    
    function spaceHandler(shift) {
        // ActionBar height: ui.du(12)
        var x = scrollView.viewableArea.x;
        var y = scrollView.viewableArea.y;
        var dy = scrollView.viewableArea.height - ui.du(18);
        if (shift)
            y -= dy;
        else
            y += dy;
        scrollView.scrollToPoint(x, y);
    }

    shortcuts: [
        Shortcut {
            key: "Space"
            onTriggered: spaceHandler(false)
        },
        Shortcut {
            key: "Shift+Space"
            onTriggered: spaceHandler(true)
        },
        Shortcut {
            key: "Backspace"
            onTriggered: {
                if (nav.count() > 1)
                    nav.pop();
            }
        },
        Shortcut {
            key: "Delete"
            onTriggered: {
                if (nav.count() > 1)
                    nav.pop();
            }
        }
    ]

    titleBar: TitleBar {
        title: root.title
    }
    
    actionBarAutoHideBehavior: ActionBarAutoHideBehavior.HideOnScroll
    actionBarVisibility: ChromeVisibility.Overlay
    
    actions: [
        ActionItem {
            id: markReadAction
            title: read == 0 ? qsTr("Mark as read") : qsTr("Mark as unread")
            ActionBar.placement: ActionBarPlacement.OnBar
            enabled: read < 2
            imageSource: read == 0 ? "asset:///read.png" : "asset:///unread.png"
            onTriggered: {
                entryModel.setData(index, "read", read == 1 ? 0 : 1, "");
                read = read == 1 ? 0 : 1;
            }
            shortcuts: Shortcut {
                key: "r"
            }
            
            onCreationCompleted: {
                if (!enabled)
                    actionSet.remove(markReadAction);
            }
        },
        ActionItem {
            title: stared ? settings.signinType < 10 ? qsTr("Unsave") : qsTr("Unstar") : settings.signinType < 10 ? qsTr("Save") : qsTr("Star") 
            ActionBar.placement: ActionBarPlacement.OnBar
            imageSource: stared ? "asset:///unsave.png" : "asset:///save.png"
            onTriggered: {
                entryModel.setData(index, "readlater", stared ? 0 : 1, "");
                stared = !stared;
            }
            shortcuts: Shortcut {
                key: "s"
            }
        },
        ActionItem {
            title: qsTr("Open in viewer")
            //enabled: settings.clickBehavior !== 0
            ActionBar.placement: ActionBarPlacement.OnBar
            imageSource: "asset:///webview.png"
            onTriggered: {
                if (check())
                    openEntryInViewer();
            }
            shortcuts: Shortcut {
                key: "v"
            }
        },
        ActionItem {
            title: qsTr("Open in browser")
            imageSource: "asset:///browser.png"
            onTriggered: {
                if (root.check()) {
                    root.openInExaternalBrowser();
                    
                    // Mark as read
                    if (read == 0) {
                        entryModel.setData(index, "read", 1, "");
                        read = 1;
                    }
                }
            }
            shortcuts: Shortcut {
                key: "Shift+v"
            }
        },
        ActionItem {
            title: qsTr("Copy")
            imageSource: "asset:///copy_link.png"
            ActionBar.placement: ActionBarPlacement.Default
            onTriggered: {
                //console.log("contents",root.contents);
                //utils.log(root.contents);
                notification.show(qsTr("URL copied to clipboard"));
                utils.copyToClipboard(root.onlineUrl);
            }
        },
        InvokeActionItem {
            title: qsTr("Share")
            ActionBar.placement: ActionBarPlacement.Default
            query {
                mimeType: "text/plain"
                invokeActionId: "bb.action.SHARE"
            }
            onTriggered: {
                data = root.onlineUrl;
            }
        },
        ActionItem {
            id: likeAction
            property bool enabled2: app.isOldReader && settings.showBroadcast
            title: root.liked ? root.isLight ? qsTr("Unlike (only in pro edition)") : qsTr("Unlike") : root.isLight ? qsTr("Like (only in pro edition)") : qsTr("Like")
            enabled: enabled2 && ! root.isLight
            imageSource: root.liked ? "asset:///unlike.png" : "asset:///like.png"
            onTriggered: {
                entryModel.setData(index, "liked", ! root.liked, "");
                root.liked = ! root.liked;
            }
            shortcuts: Shortcut {
                key: "l"
            }

            onCreationCompleted: {
                if (! enabled2)
                    root.removeAction(likeAction);
            }
        },
        ActionItem {
            id: shareAction
            property bool enabled2: app.isOldReader && settings.showBroadcast
            title: root.broadcast ? root.isLight ? qsTr("Unshare (only in pro edition)") : qsTr("Unshare") : root.isLight ? qsTr("Share (only in pro edition)") : qsTr("Share with followers")
            enabled: enabled2 && ! root.isLight
            imageSource: root.broadcast ? "asset:///unsharefollowers.png" : "asset:///sharefollowers.png"
            onTriggered: {
                if (root.broadcast) {
                    entryModel.setData(index, "broadcast", false, "");
                } else {
                    var obj = sharePage.createObject();
                    obj.index = index;
                    nav.push(obj);
                }
                root.broadcast = ! root.broadcast;
            }

            onCreationCompleted: {
                if (! enabled2)
                    root.removeAction(shareAction);
            }
        }
    ]

    Container {
        layout: DockLayout {}
        
        ScrollView {
            id: scrollView

            scrollViewProperties {
                scrollMode: ScrollMode.Vertical
                pinchToZoomEnabled: false
            }
            
            scrollRole: ScrollRole.Main
            builtInShortcutsEnabled: true

            Container {
                leftPadding: utils.du(2)
                rightPadding: utils.du(2)
                topPadding: utils.du(2)
                bottomPadding: utils.du(2)

                Label {
                    textFormat: TextFormat.Html
                    multiline: true
                    text: root.contents
                }
                
                Spacer {
                    du: 16
                }
            }
        }
        
        ViewPlaceholder {
            showIcon: false
            text: qsTr("No content")
            visible: root.empty
        }
    }
}
