/*
 * Copyright (C) 2015 Michal Kosciesza <michal@mkiol.net>
 * 
 * This file is part of Kaktus.
 * 
 * Kaktus is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Kaktus is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Kaktus.  If not, see <http://www.gnu.org/licenses/>.
 */

import bb.cascades 1.3
import net.mkiol.kaktus 1.0

Page {
    id: root
    
    property string title
    property string entryId
    property string offlineUrl
    property string onlineUrl
    property bool stared
    property bool read
    property int index
    property int feedindex
    property bool broadcast
    property string annotations
    property bool liked
    property bool cached
    property int markAsReadTime: 4000
    
    property variant g_settings: settings

    property bool readerMode: false
    property bool nightMode: false
    property bool readerModePossible: false
    property bool nightModePossible: false
    property bool scalingPossible: true
    property bool autoReaderMode: settings.readerMode

    shortcuts: [
        Shortcut {
            key: "Backspace"
            onTriggered: {
                navigateBack()
            }
        },
        Shortcut {
            key: "Space"
            onTriggered: spaceHandler(false)
        },
        Shortcut {
            key: "Shift+Space"
            onTriggered: spaceHandler(true)
        },
        Shortcut {
            key: "Delete"
            onTriggered: {
                navigateBack()
            }
        }
    ]

    actionBarAutoHideBehavior: ActionBarAutoHideBehavior.HideOnScroll
    actionBarVisibility: ChromeVisibility.Overlay
    
    onCreationCompleted: {
        Qt.isLight = utils.isLight();
        init();
    }
    
    function init() {
        navigate(settings.offlineMode ? offlineUrl : onlineUrl)
    }
    
    function disconnectSignals() {
    }
    
    function spaceHandler(shift) {
        // ActionBar height: ui.du(12)
        var scale = scrollView.contentScale;
        var x = scrollView.viewableArea.x;
        var y = scrollView.viewableArea.y;
        var dy = scrollView.viewableArea.height - ui.du(18) * (1/scale);
        if (shift)
            y -= dy;
        else
            y += dy;
        //console.log("y:", y, "scale:", scale,"height:",scrollView.viewableArea.height);
        scrollView.scrollToPoint(x, y);
    }
    
    function navigate(url) {
        view.url = url
    }
    
    function navigateBack() {
        if (view.canGoBack) {
            root.readerModePossible = false
            root.nightModePossible = false
            view.goBack()
            //root.scalingPossible = false
        } else {
            if (nav.count() > 1)
                nav.pop()
        }
    }
    
    function openUrlEntryInBrowser(url) {
        notification.show(qsTr("Launching an external browser..."));
        utils.launchBrowser(url);
    }
    
    function remAlfa(hexColor) {
        return "#" + hexColor.substring(3)
    }
    
    function initTheme() {
        var theme = { 
            "primaryColor": "#F0F0F0",
            "secondaryColor": "#E6E6E6",
            "highlightColor": remAlfa(Color.toHexString(ui.palette.primary)),
            "highlightColorDark": remAlfa(Color.toHexString(ui.palette.primaryBase)),
            "secondaryHighlightColor": remAlfa(Color.toHexString(ui.palette.primaryBase)),
            "highlightDimmerColor": "#151515",
            "fontFamily": "",
            "fontFamilyHeading": "",
            "pageMargin": ui.du(2),
            "pageMarginBottom": ui.du(8),
            "fontSize": ui.du(2),
            "fontSizeTitle": ui.du(3),
            "zoom": settings.zoom,
            "theme": settings.readerTheme }
        
        postMessage("theme_set", { "theme": theme })
    }
    
    function updateZoom(delta) {
        var zoom = settings.zoom;
        settings.zoom = ((zoom + delta) <= 0.5) || ((zoom + delta) >= 2.0) ? zoom : zoom + delta;
        var theme = { "zoom": settings.zoom };
        postMessage("theme_set", { "theme": theme });
        postMessage("theme_apply");
    }
    
    function switchReaderMode() {
        console.log("switchReaderMode")
        if (root.nightMode)
            postMessage("nightmode_disable")
        postMessage(root.readerMode ? "readability_disable" : "readability_enable")
    }
    
    function switchNightMode() {
        console.log("switchNightMode")
        postMessage(root.nightMode ? "nightmode_disable" : "nightmode_enable")
    }
    
    function messageReceivedHandler(message) {
        if (message.type === "inited") {
            // NightMode
            root.nightModePossible = true
            if ((settings.nightMode || root.nightMode) && !settings.offlineMode) {
                postMessage("nightmode_enable")
            } else {
                postMessage("nightmode_disable")
            }
            // ReaderMode
            postMessage("readability_apply_fixups")
            postMessage("readability_check", { "title": view.canGoBack ? "" : root.title });
            // Theme
            initTheme()
        } else if (message.type === "readability_result") {
            root.readerModePossible = message.data.possible
            root.readerMode = message.data.enabled
            
            // Auto switch to reader mode
            if (!root.readerMode && root.readerModePossible &&
            (root.autoReaderMode || settings.offlineMode)) {
                switchReaderMode()
                root.autoReaderMode = false
            } else if (settings.offlineMode) {
                postMessage("theme_apply")
            }
        
        } else if (message.type === "readability_status") {
            console.log("readability_status: " + message.data.enabled)
        } else if (message.type === "readability_enabled") {
            root.readerMode = true
            view.scrollToTop()
        } else if (message.type === "readability_disabled") {
            root.readerMode = false
            view.scrollToTop()
        } else if (message.type === "nightmode_enabled") {
            root.nightMode = true
        } else if (message.type === "nightmode_disabled") {
            root.nightMode = false
        }
    }
    
    function postMessage(message, data) {
        //view.postMessage(JSON.stringify({ "type": message, "data": data }));
        var objStr = JSON.stringify({ "type": message, "data": data }).replace(/[\'']/g, "\\'");
        var script = "KaktusMessageListener.onMessage('"+objStr+"');";
        console.log("postMessage: " + script);
        view.evaluateJavaScript(script);
    }
    
    paneProperties: NavigationPaneProperties {
        backButton: ActionItem {
            //imageSource: view.canGoBack ? "asset:///back-web.png" : "asset:///back.png"
            onTriggered: { navigateBack() }
        }
    }
    
    attachedObjects: [
        QTimer {
            id: timer
            singleShot: true
            interval: markAsReadTime
            onTimeout: {
                if (! root.read) {
                    root.read = true;
                    entryModel.setData(root.index, "read", 1, "");
                }
            }
        },
        ComponentDefinition {
            id: sharePage
            source: "SharePage.qml"
        }
    ]

    actions: [
        ActionItem {
            title: stared ? settings.signinType < 10 ? qsTr("Unsave") : qsTr("Unstar") : settings.signinType < 10 ? qsTr("Save") : qsTr("Star") 
            ActionBar.placement: ActionBarPlacement.OnBar
            imageSource: stared ? "asset:///unsave.png" : "asset:///save.png"
            onTriggered: {
                entryModel.setData(index, "readlater", stared ? 0 : 1, "");
                stared = !stared;
            }
            shortcuts: Shortcut {
                key: "s"
            }
        },
        ActionItem {
            id: likeAction
            property bool enabled2: app.isOldReader && settings.showBroadcast
            title: root.liked ? 
            Qt.isLight ? qsTr("Unlike (only in pro edition)") : qsTr("Unlike") : 
            Qt.isLight ? qsTr("Like (only in pro edition)") : qsTr("Like")
            enabled: enabled2 && !Qt.isLight
            imageSource: root.liked ? "asset:///unlike.png" : "asset:///like.png"
            onTriggered: {
                entryModel.setData(index, "liked", !root.liked, "");
                root.liked = !root.liked;
            }
            shortcuts: Shortcut {
                key: "l"
            }
            
            onCreationCompleted: {
                if (!enabled2)
                    root.removeAction(likeAction);
            }
        },
        InvokeActionItem {
            title: qsTr("Share")
            query {
                mimeType: "text/plain"
                invokeActionId: "bb.action.SHARE"
            }
            onTriggered: {
                data = root.onlineUrl;
            }
        },
        ActionItem {
            id: shareAction
            property bool enabled2: app.isOldReader && settings.showBroadcast
            title: root.broadcast ? 
                Qt.isLight ? qsTr("Unshare (only in pro edition)") : qsTr("Unshare") : 
                Qt.isLight ? qsTr("Share (only in pro edition)") : qsTr("Share with followers")
            enabled: enabled2 && !Qt.isLight
            imageSource: root.broadcast ? "asset:///unsharefollowers.png" : "asset:///sharefollowers.png"
            onTriggered: {
                if (root.broadcast) {
                    entryModel.setData(index, "broadcast", false, "");
                } else {
                    var obj = sharePage.createObject(); obj.index = index;
                    nav.push(obj);
                }
                root.broadcast = !root.broadcast;
            }
            
            onCreationCompleted: {
                if (!enabled2)
                    root.removeAction(shareAction);
            }
        },
        ActionItem {
            title: qsTr("Open in browser")
            imageSource: "asset:///browser.png"
            onTriggered: {
                root.openUrlEntryInBrowser(view.url)
            }
            shortcuts: Shortcut {
                key: "Alt+v"
            }
        },
        ActionItem {
            title: qsTr("Reader View")
            enabled: root.readerModePossible && !settings.offlineMode
            imageSource: root.readerMode ? "asset:///reader-selected.png" : "asset:///reader.png"
            ActionBar.placement: ActionBarPlacement.OnBar
            onTriggered: {
                root.switchReaderMode();
            }
            shortcuts: Shortcut {
                key: "r"
            }
        },
        /*ActionItem {
             title: qsTr("Night View")
             enabled: !root.readerMode && !settings.offlineMode && root.nightModePossible
             imageSource: root.nightMode ? "asset:///night-selected.png" : "asset:///night.png"
             ActionBar.placement: ActionBarPlacement.OnBar
             onTriggered: {
                 root.switchNightMode();
             }
             shortcuts: Shortcut {
                 key: "n"
             }
        },*/
        ActionItem {
            title: qsTr("Copy")
            imageSource: "asset:///copy_link.png"
            onTriggered: {
                notification.show(qsTr("URL was copied to the clipboard."));
                utils.copyToClipboard(root.onlineUrl);
            }
        },
        ActionItem {
            title: qsTr("Increase font")
            enabled: root.scalingPossible || root.readerMode
            imageSource: "asset:///fontup.png"
            onTriggered: {
                root.updateZoom(0.1)
            }
            shortcuts: Shortcut {
                key: "i"
            }
        },
        ActionItem {
            title: qsTr("Decrease font")
            enabled: root.scalingPossible || root.readerMode
            imageSource: "asset:///fontdown.png"
            onTriggered: {
                root.updateZoom(-0.1)
            }
            shortcuts: Shortcut {
                key: "d"
            }
        }
    ]

    Container {
        layout: DockLayout {}
        background: Color.Black

        ScrollView {
            id: scrollView
            
            scrollViewProperties {
                scrollMode: ScrollMode.Both
                pinchToZoomEnabled: true
            }

            shortcuts: [
                Shortcut {
                    key: "Space"
                    onTriggered: spaceHandler(false)
                },
                Shortcut {
                    key: "Shift+Space"
                    onTriggered: spaceHandler(true)
                }
            ]

            scrollRole: ScrollRole.Main
            builtInShortcutsEnabled: true

            WebView {
                id: view
                
                property bool script1Done: false
                property bool script2Done: false

                settings.viewport: {
                    "width": "device-width",
                    "initial-scale": Math.round(settings.zoom * 10 ) / 10
                }
                
                //settings.userAgent: g_settings.getDmUserAgent()
                
                function scrollToTop() {
                    scrollView.zoomToPoint(0, 0, minContentScale, ScrollAnimation.None);
                }
                
                onUrlChanged: {
                    console.log("onUrlChanged: " + url);
                    script1Done = false;
                    script2Done = false;
                }
                
                onLoadProgressChanged: {
                    //console.log("loadProgress: " + loadProgress);
                    progressIndicator.progress = loadProgress / 100.0;
                    
                    if (loadProgress > 40 && !script1Done) {
                        script1Done = true;
                        view.evaluateJavaScript(utils.readAsset("js/Kaktus.js"));
                        view.evaluateJavaScript(utils.readAsset("js/Console.js"));
                        view.evaluateJavaScript(utils.readAsset("js/MessageListener.js"));
                        view.evaluateJavaScript(utils.readAsset("js/Readability.js"));
                        view.evaluateJavaScript(utils.readAsset("js/Theme.js"));
                        view.evaluateJavaScript(utils.readAsset("js/ReaderMode.js"));
                    }
                    if (loadProgress > 90 && !script2Done) {
                        script2Done = true;
                        view.evaluateJavaScript(utils.readAsset("js/init.js"));
                    }
                }
                
                onMinContentScaleChanged: {
                    //console.log("minContentScale: " + minContentScale)
                    scrollView.scrollViewProperties.minContentScale = minContentScale;
                    scrollView.zoomToPoint(0, 0, minContentScale, ScrollAnimation.None)
                }
                
                onMaxContentScaleChanged: {
                    scrollView.scrollViewProperties.maxContentScale = maxContentScale;
                }
                  
                onLoadingChanged: {
                    if (loadRequest.status == WebLoadStatus.Started) {
                        progressIndicator.show(qsTr("Loading page content..."));
                    } else if (loadRequest.status == WebLoadStatus.Succeeded) {
                        console.log("loadRequest.status == WebLoadStatus.Succeeded")
                        progressIndicator.hide();
                        
                        // Start timer to mark as read
                        if (!root.read)
                            timer.start();
                        
                    } else if (loadRequest.status == WebLoadStatus.Failed) {
                        progressIndicator.hide();
                        
                        if (g_settings.offlineMode) {
                            notification.show(qsTr("Failed to load page from local cache."));
                        } else {
                            notification.show(qsTr("Failed to load page content."));
                        }
                        
                    }
                }
                
                onNavigationRequested: {
                    //Offile
                    if (g_settings.offlineMode) {
                        if (request.navigationType !== WebNavigationType.LinkClicked && 
                            request.navigationType !== WebNavigationType.OpenWindow) {
                            request.action = WebNavigationRequestAction.Accept;
                            return;
                        }
                        request.action = WebNavigationRequestAction.Ignore;
                        return;
                    }
                    
                    //Online
                    if (request.navigationType === WebNavigationType.LinkClicked || 
                        request.navigationType === WebNavigationType.OpenWindow) {
                        if (g_settings.webviewNavigation === 0) {
                            request.action = WebNavigationRequestAction.Ignore;
                            return;
                        }
                        
                        if (g_settings.webviewNavigation === 1) {
                            request.action = WebNavigationRequestAction.Ignore;
                            root.openUrlEntryInBrowser(request.url);
                            return;
                        }
                        
                        if (g_settings.webviewNavigation === 2) {
                            request.action = WebNavigationRequestAction.Accept;
                            root.readerMode = false;
                            root.nightMode = false;
                            root.readerModePossible = false;
                            root.nightModePossible = false;
                            return;
                        }
                    }
                }
                
                onMessageReceived: {
                    console.log("onMessageReceived data:", message.data)
                    root.messageReceivedHandler(JSON.parse(message.data))
                }
                
                onCreationCompleted: {
                    storage.clear();
                    storage.clearCookies();
                }
            }
        }

        Container {
            verticalAlignment: VerticalAlignment.Top
            horizontalAlignment: HorizontalAlignment.Left

            ProgressPanel {
                id: progressIndicator
                open: false
                onCancelClicked: {
                    view.stop();
                }
            }
        }
    }
}
